function [Statistics_post, Statistics, Cum_stats, Historic_arrival_segment, Historic_waiting_time, Historic_traversing_time, Vesselinfo] = Simulering(Replanning_threshold, NoticeSeed, DelaySeed, SlowSeed, lower_notice, upper_notice, fraction_ship_delayed, mean_delay, fraction_ship_slow, slowsegments_lower, slowsegments_upper, Visualize, T_end);

% Remember that Delays must be named Delays.txt

%% Initializations

%Start timewatch for simulation
StartSim = tic;

% Initializing time
time = 0;
last_time = 0;

% Initializing vessel information. Vesselinfo have the following columns:
% Shipnames, TGN, T_ETA, Direction, Entseg, delay, T_late, T_notice.
Vesselinfo = Generate_vesselinfo(mean_delay, fraction_ship_delayed, slowsegments_lower, slowsegments_upper, fraction_ship_slow, lower_notice, upper_notice, DelaySeed, SlowSeed, NoticeSeed);

% Initializing Moselcode
Mosel_code = moselcode;

%Initializing execution time and statistic table (from each roll)
Time_mosel = [];
Statistics = table('RowNames', {'Mosel Exec Time' 'Scheduled Ships' 'Simulation Time' 'Fixed z y/n' 'RT trigger y/n' 'Gap'});
Cum_stats = table('RowNames', {'Total travelling time' 'Average travel time' 'Max. travel time' 'Average waiting time' 'Max. single waiting time', 'Max. cumulative waiting time', 'Number of traversed ships'});

% Initializing Historic vectors
Historic_waiting_time = zeros(1, (23+1)/2);
Historic_traversing_time = zeros(1, 23);
Historic_arrival_segment = zeros(1, 23);

% Initializing LNS
list_of_new_ships = {};
rollnr = 2;
%% Creating initial plan

% Create list with ships to be scheduled
Scheduling_list = Vesselinfo(find([Vesselinfo{:,8}] <= time),:);

% Create schedule for this list
[Arrival_segment, Traversing_time, Waiting_time, Precedence, Waiting_outside, Direction, ShipNames, Time_mosel, Statistics, Scheduling_list] = Create_schedule(Scheduling_list, Mosel_code, Visualize, time, Time_mosel, Statistics, 0);

% Create list with "unscheduled" ships and "unannounced_ships"
UnScheduled_list = Vesselinfo(find([Vesselinfo{:,8}] > time),:);
Unannounced_Ships = UnScheduled_list;

%% Creating event-triggers
[T_notice, Shipname_notice] = Draw_T_notice(UnScheduled_list);
[T_ETA, Shipname_ETA, nShipsArrived] = Draw_T_ETA(Vesselinfo, 0);
[T_late, Shipname_late] = Draw_T_late(Vesselinfo, time);
[T_s, Shipname_slow] = Draw_T_s(Scheduling_list, Arrival_segment, Traversing_time, Waiting_time);

%% Simulation begins
while time < T_end
    [time, ind] = min([T_notice, T_ETA, T_late, T_s]);
    if ind == 3 && isempty(T_late)
        ind = ind + 1;
    end
    switch ind
        case 1
            %% time = T_notice
            [list_of_new_ships, Unannounced_Ships] = Update_list_of_new_ships(Unannounced_Ships, list_of_new_ships, Shipname_notice);
            if size(list_of_new_ships,1) >= Replanning_threshold
                % Replan
                Current_segment = find_ships(time, Arrival_segment, Direction);
                [Scheduling_list, UnScheduled_list, list_of_new_ships] = Create_Scheduling_list(Scheduling_list, list_of_new_ships, UnScheduled_list, Current_segment, Arrival_segment, Direction, time, 0);
                Scheduling_list = Delete_fully_traversed_ships(Scheduling_list, 0, Arrival_segment, time);
                online_mos = Fix_variables_vol3(Mosel_code, ShipNames, Scheduling_list, Arrival_segment, Traversing_time, Waiting_time, Waiting_outside, Current_segment, time);
                %online_mos = Remove_speed_limits(online_mos, Scheduling_list, Current_segment, ShipNames, time, Vesselinfo, 0);
                % Trace Historic
                [Historic_arrival_segment, Historic_waiting_time, Historic_traversing_time, last_time] = create_historic_path(time, last_time, Arrival_segment, Traversing_time, Waiting_time, ShipNames, Historic_arrival_segment, Historic_waiting_time, Historic_traversing_time);
                Historic_Waiting_outside = Calculate_historic_waiting_outside(Historic_arrival_segment, Vesselinfo);
                [Cum_stats, rollnr] = collect_cum_stat(Cum_stats, Historic_arrival_segment, Historic_waiting_time, Historic_traversing_time, Historic_Waiting_outside, rollnr, Vesselinfo);                
                [Arrival_segment, Traversing_time, Waiting_time, Precedence, Waiting_outside, Direction, ShipNames, Time_mosel, Statistics, Scheduling_list] = Create_schedule_normal(Scheduling_list, online_mos, Visualize, time, Time_mosel, Statistics, 1);
                [T_notice, Shipname_notice] = Draw_T_notice(Unannounced_Ships);
                [T_s, Shipname_slow] = Draw_T_s(Scheduling_list, Arrival_segment, Traversing_time, Waiting_time);
                disp(time);
            else
                [T_notice, Shipname_notice] = Draw_T_notice(Unannounced_Ships);
            end
        case 2
            %% time = T_ETA
            % Check if ship is in current plan
            Is_in_plan = Check_if_in_plan(Shipname_ETA, Scheduling_list);
            if Is_in_plan
                % Check if ship is late
                Is_late = Check_if_late(Shipname_ETA, Vesselinfo);
                if Is_late
                    Current_segment = find_ships(time, Arrival_segment, Direction);
                    % Remove late ship from Scheduling_list and put it on UnScheduled_list
                    [Scheduling_list, UnScheduled_list, list_of_new_ships] = Create_Scheduling_list(Scheduling_list, list_of_new_ships, UnScheduled_list, Current_segment, Arrival_segment, Direction, time, 0);
                    [Scheduling_list, UnScheduled_list, index_removed_ship] = Remove_delayed_ship(Scheduling_list, UnScheduled_list, Shipname_ETA);
                    Scheduling_list = Delete_fully_traversed_ships(Scheduling_list, 0, Arrival_segment, time);
                    online_mos = Fix_variables_vol3(Mosel_code, ShipNames, Scheduling_list, Arrival_segment, Traversing_time, Waiting_time, Waiting_outside, Current_segment, time);
                    %online_mos = Remove_speed_limits(online_mos, Scheduling_list, Current_segment, ShipNames, time, Vesselinfo, 0);
                    % Trace Historic
                    [Historic_arrival_segment, Historic_waiting_time, Historic_traversing_time, last_time] = create_historic_path(time, last_time, Arrival_segment, Traversing_time, Waiting_time, ShipNames, Historic_arrival_segment, Historic_waiting_time, Historic_traversing_time);
                    Historic_Waiting_outside = Calculate_historic_waiting_outside(Historic_arrival_segment, Vesselinfo);
                    [Cum_stats, rollnr] = collect_cum_stat(Cum_stats, Historic_arrival_segment, Historic_waiting_time, Historic_traversing_time, Historic_Waiting_outside, rollnr, Vesselinfo);
                    [Arrival_segment, Traversing_time, Waiting_time, Precedence, Waiting_outside, Direction, ShipNames, Time_mosel, Statistics, Scheduling_list] = Create_schedule_normal(Scheduling_list, online_mos, Visualize, time, Time_mosel, Statistics, 0);
                    [T_ETA, Shipname_ETA, nShipsArrived] = Draw_T_ETA(Vesselinfo, nShipsArrived);
                    [T_s, Shipname_slow] = Draw_T_s(Scheduling_list, Arrival_segment, Traversing_time, Waiting_time);
                    disp(time);
                else
                    [T_ETA, Shipname_ETA, nShipsArrived] = Draw_T_ETA(Vesselinfo, nShipsArrived);
                end
            else
                Is_late = Check_if_late(Shipname_ETA, Vesselinfo);
                if Is_late
                    list_of_new_ships = Remove_ship_from_list(list_of_new_ships, Shipname_ETA);
                    [T_ETA, Shipname_ETA, nShipsArrived] = Draw_T_ETA(Vesselinfo, nShipsArrived);
                else
                    Current_segment = find_ships(time, Arrival_segment, Direction);
                    [Scheduling_list, UnScheduled_list, list_of_new_ships] = Create_Scheduling_list(Scheduling_list, list_of_new_ships, UnScheduled_list, Current_segment, Arrival_segment, Direction, time, 0);
                    Scheduling_list = Delete_fully_traversed_ships(Scheduling_list, 0, Arrival_segment, time);
                    online_mos = Fix_variables_vol3(Mosel_code, ShipNames, Scheduling_list, Arrival_segment, Traversing_time, Waiting_time, Waiting_outside, Current_segment, time);
                    %online_mos = Remove_speed_limits(online_mos, Scheduling_list, Current_segment, ShipNames, time, Vesselinfo, 0);
                    % Trace Historic
                    [Historic_arrival_segment, Historic_waiting_time, Historic_traversing_time, last_time] = create_historic_path(time, last_time, Arrival_segment, Traversing_time, Waiting_time, ShipNames, Historic_arrival_segment, Historic_waiting_time, Historic_traversing_time);
                    Historic_Waiting_outside = Calculate_historic_waiting_outside(Historic_arrival_segment, Vesselinfo);
                    [Cum_stats, rollnr] = collect_cum_stat(Cum_stats, Historic_arrival_segment, Historic_waiting_time, Historic_traversing_time, Historic_Waiting_outside, rollnr, Vesselinfo);
                    [Arrival_segment, Traversing_time, Waiting_time, Precedence, Waiting_outside, Direction, ShipNames, Time_mosel, Statistics, Scheduling_list] = Create_schedule_normal(Scheduling_list, online_mos, Visualize, time, Time_mosel, Statistics, 0);
                    [T_ETA, Shipname_ETA, nShipsArrived] = Draw_T_ETA(Vesselinfo, nShipsArrived);
                    [T_s, Shipname_slow] = Draw_T_s(Scheduling_list, Arrival_segment, Traversing_time, Waiting_time);
                    disp(time);
                end
            end
            
        case 3
            %% time = T_late
            % Replan
            Current_segment = find_ships(time, Arrival_segment, Direction);
            % Have to put the late ship on the list
            [Scheduling_list, UnScheduled_list] = Add_delayed_ship(Scheduling_list, UnScheduled_list, Shipname_late);
            [Scheduling_list, UnScheduled_list, list_of_new_ships] = Create_Scheduling_list(Scheduling_list, list_of_new_ships, UnScheduled_list, Current_segment, Arrival_segment, Direction, time, Shipname_late);
            Scheduling_list = Delete_fully_traversed_ships(Scheduling_list, 0, Arrival_segment, time);
            online_mos = Fix_variables_vol3(Mosel_code, ShipNames, Scheduling_list, Arrival_segment, Traversing_time, Waiting_time, Waiting_outside, Current_segment, time);
            %online_mos = Remove_speed_limits(online_mos, Scheduling_list, Current_segment, ShipNames, time, Vesselinfo, 0);
            % Trace Historic
            [Historic_arrival_segment, Historic_waiting_time, Historic_traversing_time, last_time] = create_historic_path(time, last_time, Arrival_segment, Traversing_time, Waiting_time, ShipNames, Historic_arrival_segment, Historic_waiting_time, Historic_traversing_time);
            Historic_Waiting_outside = Calculate_historic_waiting_outside(Historic_arrival_segment, Vesselinfo);
            [Cum_stats, rollnr] = collect_cum_stat(Cum_stats, Historic_arrival_segment, Historic_waiting_time, Historic_traversing_time, Historic_Waiting_outside, rollnr, Vesselinfo);
            [Arrival_segment, Traversing_time, Waiting_time, Precedence, Waiting_outside, Direction, ShipNames, Time_mosel, Statistics, Scheduling_list] = Create_schedule_normal(Scheduling_list, online_mos, Visualize, time, Time_mosel, Statistics, 0);
            [T_late, Shipname_late] = Draw_T_late(Vesselinfo, time);
            [T_s, Shipname_slow] = Draw_T_s(Scheduling_list, Arrival_segment, Traversing_time, Waiting_time);
            disp(time);
        case 4
            %% time = T_slow
            Current_segment = find_ships_slow(time, Arrival_segment, Direction);
            [Scheduling_list, UnScheduled_list, ~] = Create_Scheduling_list(Scheduling_list, {}, UnScheduled_list, Current_segment, Arrival_segment, Direction, time, 0);
            Scheduling_list = Delete_fully_traversed_ships(Scheduling_list, 0, Arrival_segment, time);
            online_mos = Fix_variables_vol3(Mosel_code, ShipNames, Scheduling_list, Arrival_segment, Traversing_time, Waiting_time, Waiting_outside, Current_segment, time);
            %online_mos = Remove_speed_limits(online_mos, Scheduling_list, Current_segment, ShipNames, time, Vesselinfo, Shipname_slow);
            online_mos = Check_and_remove_aligned_conflict(online_mos, Scheduling_list, time, Shipname_slow, Arrival_segment, ShipNames);
            [Historic_arrival_segment, Historic_waiting_time, Historic_traversing_time, last_time] = create_historic_path(time, last_time, Arrival_segment, Traversing_time, Waiting_time, ShipNames, Historic_arrival_segment, Historic_waiting_time, Historic_traversing_time);
            Historic_Waiting_outside = Calculate_historic_waiting_outside(Historic_arrival_segment, Vesselinfo);
            [Cum_stats, rollnr] = collect_cum_stat(Cum_stats, Historic_arrival_segment, Historic_waiting_time, Historic_traversing_time, Historic_Waiting_outside, rollnr, Vesselinfo);
            try
                [Arrival_segment, Traversing_time, Waiting_time, Precedence, Waiting_outside, Direction, ShipNames, Time_mosel, Statistics, Scheduling_list] = Update_schedule_w_fixed_z(Scheduling_list, online_mos, Visualize, time, Time_mosel, Statistics, Shipname_slow, Precedence, ShipNames, 0);
            catch
                [Arrival_segment, Traversing_time, Waiting_time, Precedence, Waiting_outside, Direction, ShipNames, Time_mosel, Statistics, Scheduling_list] = Update_schedule_w_fixed_z(Scheduling_list, online_mos, Visualize, time, Time_mosel, Statistics, Shipname_slow, Precedence, ShipNames, 1);
            end
            Scheduling_list = Remove_slowcase_trigger(Scheduling_list, Shipname_slow);
            [T_s, Shipname_slow] = Draw_T_s(Scheduling_list, Arrival_segment, Traversing_time, Waiting_time);
            disp(time);
    end
end
EndSim = toc(StartSim);

%% Post-processing
[Historic_arrival_segment, Historic_waiting_time, Historic_traversing_time, last_time] = create_historic_path(time, last_time, Arrival_segment, Traversing_time, Waiting_time, ShipNames, Historic_arrival_segment, Historic_waiting_time, Historic_traversing_time);
Historic_Waiting_outside = Calculate_historic_waiting_outside(Historic_arrival_segment, Vesselinfo);
[Direction, Historic_traffic_group_number, Historic_ShipNames] = Extract_vesselinfo(Vesselinfo, Scheduling_list);
[Kiel_length, Passage_nr] = Gather_Canal_topology();
Passage_nr = int32(Passage_nr');
f3 = figure;
Draw_schedule(Historic_waiting_time, Historic_arrival_segment, Direction, Kiel_length, Historic_traversing_time, Historic_traffic_group_number, Passage_nr, Historic_ShipNames, time);
Statistics;
Statistics_post = Collect_statistics_post(Historic_arrival_segment, Historic_traversing_time, Historic_waiting_time, Historic_Waiting_outside, Vesselinfo, EndSim)


end

