function [Arrival_segment, Traversing_time, Waiting_time, Precedence, Waiting_outside, Direction, ShipNames] = Create_schedule(Scheduling_list, Mosel_code, Visualize, time)

%% Gathering information about canal topology
[Kiel_length, Passage_number] = Gather_Canal_topology();
%% Gather simple Mosel inputs
nSegments = size(Kiel_length,2)-1;
Segments = 1:nSegments;
nSidings = (nSegments+1)/2;
Sidings = 1:2:nSegments;
nTransits = (nSegments-1)/2;
Transits = 2:2:nSegments;
nShips = size(Scheduling_list,1);
Ships = 1:nShips;
ShipNames = cell(size(Scheduling_list,1),1);
Traffic_group_number = zeros(size(Scheduling_list,1),1);
ETA = zeros(size(Scheduling_list,1),1);
Direction = zeros(size(Scheduling_list,1),1);
ShipSegments = cell(nShips,1);
for i = 1:size(Scheduling_list,1)
    ShipNames{i} = Scheduling_list{i,1};
    Traffic_group_number(i) = Scheduling_list{i,2};
    ETA(i) = Scheduling_list{i,3};
    Direction(i) = Scheduling_list{i,4};
    EnteringSegment(i) = Scheduling_list{i,5};
    if Direction(i) == 1
        ShipSegments{i,1} = EnteringSegment(i):nSegments;
    elseif Direction(i) == -1
        ShipSegments{i,1} = EnteringSegment(i):-1:1;
    end
end
Ships_W = Ships(Direction == -1);
Ships_E = Ships(Direction == 1);

%% Calculating complex Mosel inputs
VesselSpeed = create_Vesselspeed(Traffic_group_number);
MinimumTraversingTime = Calculate_TraversingTime(VesselSpeed, ShipSegments, Kiel_length);
EarliestSegmentTime = Calculate_EarliestSegmentTime(MinimumTraversingTime, ETA, ShipSegments, Direction);
[FastestSegmentTime, SlowestSegmentTime] = create_speed_limits(MinimumTraversingTime, Traffic_group_number);
SafetyTime = create_SafetyTime(VesselSpeed, Traffic_group_number, Direction);
M = nShips*max(sum(MinimumTraversingTime,2));
ConflictingShips = create_conflict(Direction, Traffic_group_number, Passage_number, ShipSegments);
[ShipTransits, ShipSidings] = Split_ShipSegments(ShipSegments);

%% Writing ShipTransits, ShipSidings and ShipSegments to file
Mosel_file = writeFile(ShipSegments, ShipSidings, ShipTransits);

%% Executing Moselcode, creating schedule
initial_mos = Mosel_code;
retcode = moselexec('matlab.mws:initial_mos');
if retcode ~= 0
    msgbox('Mosel did not run properly', 'Error message')
end
Waiting_time(:,2:2:size(Waiting_time,2)) = [];

%% Visualize schedule
% Visualize plan
if Visualize
    f1 = figure;
    Draw_schedule(Waiting_time, Arrival_segment, Direction, Kiel_length, Traversing_time, Traffic_group_number, Passage_number, ShipNames, time);

end
end