function [Arrival_segment, Traversing_time, Waiting_time, Precedence, Waiting_outside, Direction, ShipNames, Time_mosel, Statistics, Scheduling_list] = Create_schedule_weighting(Scheduling_list, Mosel_code, Visualize, time, Time_mosel, Statistics, Shipname_slow, Precedence, ShipNames_prev)

%% Gathering information about canal topology
[Kiel_length, Passage_number] = Gather_Canal_topology();
%% Gather simple Mosel inputs
nSegments = size(Kiel_length,2)-1;
Segments = 1:nSegments;
nSidings = (nSegments+1)/2;
Sidings = 1:2:nSegments;
nTransits = (nSegments-1)/2;
Transits = 2:2:nSegments;
nShips = size(Scheduling_list,1);
Ships = 1:nShips;
ShipNames = cell(size(Scheduling_list,1),1);
Traffic_group_number = zeros(size(Scheduling_list,1),1);
ETA = zeros(size(Scheduling_list,1),1);
Direction = zeros(size(Scheduling_list,1),1);
ShipSegments = cell(nShips,1);
for i = 1:size(Scheduling_list,1)
    ShipNames{i} = Scheduling_list{i,1};
    Traffic_group_number(i) = Scheduling_list{i,2};
    ETA(i) = Scheduling_list{i,3};
    Direction(i) = Scheduling_list{i,4};
    EnteringSegment(i) = Scheduling_list{i,5};
    if Direction(i) == 1
        ShipSegments{i,1} = EnteringSegment(i):nSegments;
    elseif Direction(i) == -1
        ShipSegments{i,1} = EnteringSegment(i):-1:1;
    end
end
Ships_W = Ships(Direction == -1);
Ships_E = Ships(Direction == 1);

%% Calculating complex Mosel inputs
if Shipname_slow == 0
    VesselSpeed = create_Vesselspeed(Traffic_group_number);
    MinimumTraversingTime = Calculate_TraversingTime(VesselSpeed, ShipSegments, Kiel_length);
    EarliestSegmentTime = Calculate_EarliestSegmentTime(MinimumTraversingTime, ETA, ShipSegments, Direction);
    [FastestSegmentTime, SlowestSegmentTime] = create_speed_limits(MinimumTraversingTime, Traffic_group_number);
    SafetyTime = create_SafetyTime(VesselSpeed, Traffic_group_number, Direction);
    [WeightingArray, WeightOutside] = createWeightingArray(nShips, nSidings, Direction);
    M = nShips*max(sum(MinimumTraversingTime,2));
    ConflictingShips = create_conflict(Direction, Traffic_group_number, Passage_number, ShipSegments);
    [ShipTransits, ShipSidings] = Split_ShipSegments(ShipSegments);
else
    VesselSpeed = create_Vesselspeed(Traffic_group_number);
    MinimumTraversingTime = Calculate_TraversingTime(VesselSpeed, ShipSegments, Kiel_length);
    MinimumTraversingTime = Slow_down(MinimumTraversingTime, Shipname_slow, Scheduling_list, Kiel_length, VesselSpeed);
    EarliestSegmentTime = Calculate_EarliestSegmentTime(MinimumTraversingTime, ETA, ShipSegments, Direction);
    [FastestSegmentTime, SlowestSegmentTime] = create_speed_limits(MinimumTraversingTime, Traffic_group_number);
    [FastestSegmentTime, SlowestSegmentTime] = fix_slow_speed(FastestSegmentTime, SlowestSegmentTime, Scheduling_list, Shipname_slow);
    SafetyTime = create_SafetyTime(VesselSpeed, Traffic_group_number, Direction);
    M = nShips*max(sum(MinimumTraversingTime,2));
    ConflictingShips = create_conflict(Direction, Traffic_group_number, Passage_number, ShipSegments);
    [ShipTransits, ShipSidings] = Split_ShipSegments(ShipSegments);
    [WeightingArray, WeightOutside] = createWeightingArray(nShips, nSidings, Direction);
    Mosel_code = Remove_speed_fix(Mosel_code, ShipNames_prev, Shipname_slow);
    Mosel_code = Fix_precedence(Precedence, Mosel_code, Scheduling_list, ConflictingShips, ShipNames_prev);
end
%% Writing ShipTransits, ShipSidings and ShipSegments to file
Mosel_file = writeFile(ShipSegments, ShipSidings, ShipTransits);

%% Executing Moselcode, creating schedule
initial_mos = Mosel_code;
tic;
retcode = moselexec('matlab.mws:initial_mos');
Time_mosel = [Time_mosel, toc];

if retcode ~= 0 || sum(sum(Arrival_segment)) == 0
    error('Mosel did not run properly, might be due to infeasibilities');
end
fixed_flag = 0;
RT_flag = 0;
Statistics = Collect_statistics(Statistics, Time_mosel, Scheduling_list, time, fixed_flag, RT_flag, Gap);
Waiting_time(:,2:2:size(Waiting_time,2)) = [];

%% Visualize schedule
% Visualize plan
if Visualize
    f1 = figure;
    Draw_schedule(Waiting_time, Arrival_segment, Direction, Kiel_length, Traversing_time, Traffic_group_number, Passage_number, ShipNames, time);
end
end